//
// DEC97.CPP
//
// Visual C++ 5.0 still has a few problems dealing with
// nested classes.  In this simple program, we have two nested
// classes that happen to share the same name.  Class FooBar
// is derived from both of these classes, which works
// out fine, because the names are fully qualified with
// the name of the enclosing classes.
//
// When I invoke foo() directly from an object of class FooBar,
// the correct version of foo() is called, and the string
// "foo" is returned.  But oddly enough, when I call foo()
// from the same object after having been upcast to
// type Base1::Nested, the wrong function gets called, and
// string "bar" is returned!
//
// It seems as though the compiler is getting confused by 
// the fact that the two base classes both have the same 
// name.  Changing the inner class names so they are no 
// longer identical fixes the problem.
//
// Both Watcom 11.0 and Borland C++ Builder are able to
// execute this program without the unexpected results.
//
// Program output:
//
//  c.foo() returns: foo
//  c.bar() returns: bar
//  p_upcast->foo() returns: bar
//
#include <iostream>

using namespace std;

class Base1 {
public:
  class Nested {
  public :
    virtual char *foo() = 0;
  };
};

class Base2 {
public:
  class Nested {
  public :
    virtual char *bar() = 0;
  };
};

class FooBar : public Base1::Nested, public Base2::Nested {
public:
  char *foo() { return "foo"; }
  char *bar() { return "bar"; }
};

void main()
{
  FooBar c;
  Base1::Nested* p_upcast_c = &c;
  cout << "c.foo() returns: " << c.foo() << "\n";
  cout << "c.bar() returns: " << c.bar() << "\n";
  cout << "p_upcast->foo() returns: " << p_upcast_c->foo() << "\n";
}
